@@ -179,7 +179,8 @@ class Artifact(object):
 
     def setup_from_filter_class(self):
         # cache filter class source code so it only has to be calculated once
-        if not hasattr(self.filter_class, 'SOURCE_CODE'):
+        filter_class_source_const = "SOURCE_CODE_%s" % self.filter_class.__name__
+        if not hasattr(self.filter_class, filter_class_source_const):
             # get source code of this filter class + all parent filter classes.
             source = ""
             klass = self.filter_class
@@ -192,15 +193,17 @@ class Artifact(object):
             # and then get source code of DexyFilter class
             source += inspect.getsource(dexy.dexy_filter.DexyFilter)
 
-            filter_class_source = source
-            self.filter_class.SOURCE_CODE = self.compute_hash(filter_class_source)
+            filter_class_source_hash = self.compute_hash(source)
+            setattr(self.filter_class, filter_class_source_const, filter_class_source_hash)
+            assert filter_class_source_hash == getattr(self.filter_class, filter_class_source_const)
+            self.log.debug("Source code hash for %s is %s" % (self.filter_class.__name__, filter_class_source_hash))
 
         if not hasattr(self.filter_class, 'VERSION'):
             filter_version = self.filter_class.version(self.log)
             self.filter_class.VERSION = filter_version
 
         self.filter_name = self.filter_class.__name__
-        self.filter_source = self.filter_class.SOURCE_CODE
+        self.filter_source = getattr(self.filter_class, filter_class_source_const)
         self.filter_version = self.filter_class.VERSION
 
         if self.final is None:
@@ -217,7 +220,7 @@ class Artifact(object):
             if a.additional and not k in self._inputs:
                 self.log.debug("(%s) Adding additional artifact %s from %s" % (self.key, k, a.key))
                 self.add_input(k, a)
-            elif not k in self._inputs:
+            elif not k in self._inputs and not a.virtual:
                 # We should have all other inputs already. Validate this.
                 raise Exception("Missing input %s" % k)
 
@@ -229,6 +232,7 @@ class Artifact(object):
         self.binary_input = previous_artifact.binary_output
         self.input_data_dict = previous_artifact.data_dict
         self.input_ext = previous_artifact.ext
+        self.previous_artifact_hashstring = previous_artifact.hashstring
         self.previous_artifact_filename = previous_artifact.filename()
         self.previous_artifact_filepath = previous_artifact.filepath()
         self.previous_canonical_filename = previous_artifact.canonical_filename(True)
@@ -296,7 +300,6 @@ class Artifact(object):
 
         # Set inputs from original document inputs.
         artifact._inputs.update(artifact.doc.input_artifacts())
-        artifact.log.debug("add inputs from document %s: %s" % (artifact.doc.key(), ", ".join(artifact.doc.input_artifacts().keys())))
 
         for k, a in artifact.doc.input_artifacts().iteritems():
             if a.additional and not k in artifact._inputs:
@@ -317,7 +320,6 @@ class Artifact(object):
         return artifact
 
     def run(self):
-
         start = time.time()
 
         if self.controller_args['nocache'] or not self.is_complete():
@@ -330,6 +332,12 @@ class Artifact(object):
             filter_instance.artifact = self
             filter_instance.log = self.log
 
+            # Make sure previous artifact is loaded.
+            if not self.binary_input and len(self.input_text()) == 0:
+                f = open(self.previous_artifact_filepath, "rb")
+                self.data_dict['1'] = f.read()
+                f.close()
+
             try:
                 filter_instance.process()
             except Exception as e:
@@ -376,8 +384,10 @@ class Artifact(object):
         self.elapsed = time.time() - start
         self.db.update_artifact(self)
 
-    def add_additional_artifact(self, key_with_ext, ext):
+    def add_additional_artifact(self, key_with_ext, ext=None):
         """create an 'additional' artifact with random hashstring"""
+        if not ext:
+            ext = os.path.splitext(key_with_ext)[1]
         new_artifact = self.__class__()
         new_artifact.key = key_with_ext
         if ext.startswith("."):
@@ -400,6 +410,7 @@ class Artifact(object):
                 setattr(new_artifact, at, val)
 
         new_artifact.set_hashstring()
+        self.log.debug("new artifact %s hashstring %s" % (key_with_ext, new_artifact.hashstring))
         self.add_input(key_with_ext, new_artifact)
         self.db.append_artifact(new_artifact) # append to db because not part of doc.artifacts
         return new_artifact
@@ -556,6 +567,17 @@ class Artifact(object):
     def output_text(self):
         return u"".join([self.convert_if_not_unicode(v) for k, v in self.data_dict.items()])
 
+    def output(self):
+        if not self.is_complete():
+            raise Exception("can't call output unless complete!")
+
+        if self.binary_output:
+            if not hasattr(self, 'binary_data'):
+                self.load_output()
+            return self.binary_data
+        else:
+            return self.output_text()
+
     def relative_refs(self, relative_to_file):
         """How to refer to this artifact, relative to another."""
 
