@@ -97,7 +97,6 @@ class FileSystemJsonArtifact(Artifact):
     def save_output(self):
         if not self.is_complete():
             raise Exception("should not be calling save_output unless artifact is complete")
-
         if not self.binary_output:
             if not self.data_dict or len(self.data_dict) == 0:
                 # Our filter has written directly to an output file
@@ -118,8 +117,10 @@ class FileSystemJsonArtifact(Artifact):
     def load_output(self):
         if not self.is_complete():
             raise Exception("should not be calling load_output unless artifact is complete")
-
         if not self.binary_output:
             output_dict = self.read_dict_from_file(self.cached_output_filepath())
             self.data_dict = self.convert_numbered_dict_to_ordered_dict(output_dict)
+        else:
+            with open(self.filepath(), "rb") as f:
+                self.binary_data = f.read()
 
