@@ -22,6 +22,7 @@ class ProcessFilter(DexyFilter):
     """
     ALIASES = ['processfilter']
     CHECK_RETURN_CODE = True
+    ENV = None
 
     def ignore_errors(self):
         """
@@ -47,13 +48,19 @@ class ProcessFilter(DexyFilter):
                 raise DexyNonzeroExitException(command, exitcode, stderr)
 
     def setup_env(self):
+        env = os.environ
+        if self.ENV:
+            env.update(self.ENV)
         if self.artifact.args.has_key('env'):
-            env = os.environ
             env.update(self.artifact.args['env'])
-        else:
-            env = None
         return env
 
+    def setup_initial_timeout(self):
+        if self.INITIAL_PROMPT_TIMEOUT:
+            return self.INITIAL_PROMPT_TIMEOUT
+        else:
+            return 3
+
     def setup_timeout(self):
         if self.artifact.args.has_key('timeout'):
             timeout = self.artifact.args['timeout']
@@ -146,7 +153,6 @@ class SubprocessFilter(ProcessFilter):
             self.log.debug("about to send input '%s'" % input_text)
 
         stdout, stderr = proc.communicate(input_text)
-        self.log.debug(stdout)
         return (proc, stdout)
 
 class SubprocessStdoutFilter(SubprocessFilter):
@@ -157,8 +163,7 @@ class SubprocessStdoutFilter(SubprocessFilter):
     def process(self):
         command = self.command_string_stdout()
         proc, stdout = self.run_command(command, self.setup_env())
-        if self.CHECK_RETURN_CODE:
-            self.handle_subprocess_proc_return(command, proc.returncode, stdout)
+        self.handle_subprocess_proc_return(command, proc.returncode, stdout)
         self.artifact.set_data(stdout)
 
 class SubprocessStdoutInputFilter(SubprocessFilter):
